/*
 y = levelSet2DTI(x,gamma,delta,penWeight);
 */

#include <math.h>
#include <stdio.h>
#include "mex.h"
#include "matrix.h"
#include "mathHelper.c"
#include "pruneTree2DTIMulti.c"

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
  double *y, *S;
  double **y2, **S2;
  int im1, im2, m1,m2, K, k;
  int i, j, L, nDim, nElts;
  const int *dims;
  double mtest, ntest, delta, *gamma, penWeight, sum = 0;
  char *noiseType;
	
  /* check for correct # of input variables */
  if (nrhs>4){
    mexErrMsgTxt("There are at most 4 input parameters allowed!");
    return;
  }
  if (nrhs<2){
    mexErrMsgTxt("There are at least 2 input parameters required!");
    return;
  }
	
  y = mxGetPr(prhs[0]);
  nDim = mxGetNumberOfDimensions(prhs[0]);
  dims = mxGetDimensions(prhs[0]);
  nElts = mxGetNumberOfElements(prhs[0]);
	
  if (nDim > 2) {
    mexErrMsgTxt("The input array cannot have more than two dimensions.");
    return;
  }
	
  /* Check the dimensions of input */
  for (i = 0; i < nDim; i++) {
    mtest = (double) log((mxGetDimensions(prhs[0]))[i])/log(2.0);
    if (!isint(mtest)) {
      mexErrMsgTxt("Each matrix dimension must be of size m*2^(L)");
    }
  }
	
  /* set noiseType */
  noiseType = "Unif";
	
  /* set gamma */
  if ((nrhs < 2)||(mxGetN(prhs[1])==0)) {
    mexErrMsgTxt("You must input the level(s) gamma.");
  }
  else {
    gamma = mxGetPr(prhs[1]);
    K = mxGetNumberOfElements(prhs[1]);
  }

	
  /* set delta */
  if ((nrhs < 3)||(mxGetN(prhs[2])==0)) {
    delta = 1/(double)nElts;
  }
  else {
    delta = (double) *mxGetPr(prhs[2]);    
  }
  if (delta < 0)
    mexErrMsgTxt("The error probability delta must be positive.");
	
  /* set penalty */
  if ((nrhs < 4)||(mxGetN(prhs[3])==0)) {
    penWeight = 1;
  }
  else {
    penWeight = (double) *mxGetPr(prhs[3]);
  }
  if (penWeight < 0)
    mexErrMsgTxt("The penalty weight must be positive.");
	
  plhs[0] = mxCreateNumericArray(nDim,dims,mxDOUBLE_CLASS,mxREAL);
  S = mxGetPr(plhs[0]);
  if (nDim==1) {
    mexErrMsgTxt("This has not been implemented for 1D");
  }
  else if (nDim==2) {
    m1 = (int)(mxGetDimensions(prhs[0]))[0];
    m2 = (int)(mxGetDimensions(prhs[0]))[1];
    if (m1==1) {
      mexErrMsgTxt("This has not been implemented for 1D");
    }
    else if (m2==1) {
      mexErrMsgTxt("This has not been implemented for 1D");
    }
    else {
      y2 = (double **)mxCalloc(m1,sizeof(double*));
      S2 = (double **)mxCalloc(m1,sizeof(double*));
      for (im1 = 0; im1 < m1; im1++) {
	y2[im1] = (double *)mxCalloc(m2,sizeof(double));
	S2[im1] = (double *)mxCalloc(m2,sizeof(double));
	for (im2 = 0; im2 < m2; im2++) {
	  y2[im1][im2] = y[im1+im2*m1];
	}
      }
      pruneTree2DTIMulti(y2, m1, m2, gamma, K, delta, 
			 penWeight, S2, noiseType);
      for (im1 = 0; im1 < m1; im1++) {
	for (im2 = 0; im2 < m2; im2++) {
	  S[im1+im2*m1] = S2[im1][im2];
	}
      }
    }
  }
  else {
    mexErrMsgTxt("Too many dimensions");
  }
}
