void pruneTree2DTIMulti(double **y, int m, int n, double *gamma, 
			int K, double delta, double penWeight, 
			double **S, char *noiseType)
{
  int L,in,im,iL,j,kn,km,iL2, dyadLen, k, obsPerDyad;
  double ***SScaling;
  double **Offset, **Offset_prev, **cost, **cost_prev;
  double ***waveletScale, ***splitDecision;
  double xD1, xD2, xD3, xD4, Offset_k;
  double qA, pA, mergeCost, splitCost;
	
  L = (int) floor(log(min(m,n))/log((double)2));
	
  Offset = (double **)mxCalloc(m,sizeof(double*));
  Offset_prev = (double **)mxCalloc(m,sizeof(double*));
  cost = (double **)mxCalloc(m,sizeof(double*));
  cost_prev = (double **)mxCalloc(m,sizeof(double*));
	
  SScaling = (double ***)mxCalloc(m,sizeof(double**));
  waveletScale = (double ***)mxCalloc(m,sizeof(double**));
  splitDecision = (double ***)mxCalloc(m,sizeof(double**));

  for (im = 0; im < m; im++) {
    Offset[im] = (double *)mxCalloc(n,sizeof(double));
    Offset_prev[im] = (double *)mxCalloc(n,sizeof(double));
    cost[im] = (double *)mxCalloc(n,sizeof(double));
    cost_prev[im] = (double *)mxCalloc(n,sizeof(double));
		
    SScaling[im] = (double **)mxCalloc(n,sizeof(double*));
    waveletScale[im] = (double **)mxCalloc(n,sizeof(double*));
    splitDecision[im] = (double **)mxCalloc(n,sizeof(double*));
		
    for (in = 0; in <n; in++) {
      SScaling[im][in] = (double *)mxCalloc(L+1,sizeof(double));
      waveletScale[im][in] = (double *)mxCalloc(L,sizeof(double));
      splitDecision[im][in] = (double *)mxCalloc(L,sizeof(double));
    }
  }

  /* initialize variables */
  qA = 3*L+2;
  pA = 4*max(1, (log(1/delta)+qA*log(2)))/m/n;

  for (im = 0; im < m; im++) {
    for (in = 0; in < n; in++){
      S[im][in] = 0;
      cost_prev[im][in] = penWeight*pow(2*pA*(log(2/delta)+qA*log(2))/m/n,0.5);
      Offset_prev[im][in] = y[im][in]/(double)m/(double)n/(double)K/2.0;
      for (k=0; k < K; k++) {
	Offset_k = gamma[k]/(double)m/(double)n/(double)K/2.0 - Offset_prev[im][in];
	S[im][in] += (double) (Offset_k<0)/(double)K;
	cost_prev[im][in] += Offset_k*(2.0*(Offset_k<0)-1.0);
      }
      SScaling[im][in][0] = S[im][in];
    }
  }


  for (iL=0;iL<L;iL++) {
    dyadLen = (int)pow(2.0,iL+1);
    obsPerDyad = (int)pow(dyadLen,2.0);
    /*printf("dyadLen = %d, il = %d, L = %d, obsPerDyad = %d\n",dyadLen,iL,L,obsPerDyad);*/
    qA = 3*(L-iL)+2;
    pA = pow(dyadLen,2.0)/m/n;
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++){
        km = (int)mod(im+dyadLen/2,m);
        kn = (int)mod(in+dyadLen/2,n);
				
	splitCost = cost_prev[im][in]+cost_prev[km][in]+
	  cost_prev[im][kn]+cost_prev[km][kn];
	
	mergeCost = penWeight*pow(2*pA*(log(2/delta)+qA*log(2))/m/n,0.5);
	SScaling[im][in][iL+1]=0;
	Offset[im][in] = Offset_prev[im][in]+Offset_prev[km][in]+
	  Offset_prev[im][kn]+Offset_prev[km][kn];
	
	for (k=0; k < K; k++) {
	  Offset_k = gamma[k]/(double)m/(double)n/(double)K/2.0*(double)obsPerDyad - Offset[im][in];
	  
	  mergeCost += Offset_k*(2.0*(Offset_k<0)-1.0);
	  SScaling[im][in][iL+1] += (Offset_k<0)/(double)K; 
	}
        if (splitCost > mergeCost ) {
	  cost[im][in] = mergeCost;
          splitDecision[im][in][iL] = 0.0;
	}
        else {
	  cost[im][in] = splitCost;
          splitDecision[im][in][iL] = 1.0;
	}
	waveletScale[im][in][iL] = 1.0;
      }
    }
    
    
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++) {
	cost_prev[im][in] = cost[im][in];
	Offset_prev[im][in] = Offset[im][in];
      }
    }
  }
  for (iL=L-1; iL>=0; iL--) {
    dyadLen = (int)pow(2.0,iL+1);
    
    for (in = 0; in < n; in++) {
      for (im = 0; im < m; im++) {
        km = (int)mod(im+dyadLen/2,m);
        kn = (int)mod(in+dyadLen/2,n);
        if (splitDecision[im][in][iL]==0) {
          waveletScale[im][in][iL] = 0.0;
        }
	/*printf("waveletScale[%d][%d][%d]=%f\n",im,in,iL,waveletScale[im][in][iL]);*/
				
        if (iL > 0) {
          waveletScale[im][in][iL-1] = waveletScale[im][in][iL-1]
	    - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[im][kn][iL-1] = waveletScale[im][kn][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[km][in][iL-1] = waveletScale[km][in][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
          waveletScale[km][kn][iL-1] = waveletScale[km][kn][iL-1]
            - (0.25*(1.0-waveletScale[im][in][iL]));
        }
      }
    }
		
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++) {
	kn = (int)mod(in-dyadLen/2,n);
        km = (int)mod(im-dyadLen/2,m);
	
	/*printf("iL=%d\tdyadLen=%d\t[im,in]=[%d,%d]\t[km,kn]=[%d,%d]\n",iL,dyadLen,im,in,km,kn);
	  printf("waveletScale[%d][%d][%d]=%f\n",iL,im,in,waveletScale[im][in][iL]);*/
        xD1 = SScaling[im][in][iL]*waveletScale[im][in][iL]+
	  S[im][in]*(1.0-waveletScale[im][in][iL]);
        xD2 = SScaling[im][in][iL]*waveletScale[im][kn][iL]+
	  S[im][kn]*(1.0-waveletScale[im][kn][iL]);
        xD3 = SScaling[im][in][iL]*waveletScale[km][in][iL]+
	  S[km][in]*(1.0-waveletScale[km][in][iL]);
        xD4 = SScaling[im][in][iL]*waveletScale[km][kn][iL]+
	  S[km][kn]*(1.0-waveletScale[km][kn][iL]);
	
	Offset[im][in] = (xD1 + xD2 + xD3 + xD4)/4.0;
      }
    }
    /*printf("iL=%d, Scaling Coeffs = \n",iL);*/
    for (im = 0; im < m; im++) {
      for (in = 0; in < n; in++) {
        S[im][in] = Offset[im][in];
	/*printf("%f\t",S[im][in]);*/
      }
      /*printf("\n");*/
    }
  }
  for (im = 0; im < m; im++) {
    for (in = 0; in < n; in++) {
      S[im][in] = (round(S[im][in]*(double)K)/((double)K));
    }
  }

}
